package com.raulvibo.tester;

import android.support.v7.widget.RecyclerView;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;



import java.util.Calendar;
import java.util.List;
import java.util.Locale;

/**
 * Created by raulvibo on 30/08/2017.
 */

public class AdaptadorFichaTest extends RecyclerView.Adapter<AdaptadorFichaTest.FichaTestViewHolder> {
    List<FichaTest> fichasTestsActivos;
    List<FichaTest> fichasTestsCreador;
    View.OnClickListener onClickListener;
    View.OnLongClickListener onLongClickListener;
    boolean filtroMisFichas = false;

    public AdaptadorFichaTest(List<FichaTest> fichasTestsActivos, List<FichaTest> fichasTestsCreador) {
        this.fichasTestsActivos = fichasTestsActivos;
        this.fichasTestsCreador = fichasTestsCreador;
    }

    @Override
    public FichaTestViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.elemento_listado, parent, false);
        view.setOnClickListener(onClickListener);
        view.setOnLongClickListener(onLongClickListener);
        FichaTestViewHolder fichaTestViewHolder = new FichaTestViewHolder(view);
        return fichaTestViewHolder;
    }

    public boolean isFiltroMisFichas() {
        return filtroMisFichas;
    }

    public void setFiltroMisFichas(boolean filtroMisFichas) {
        this.filtroMisFichas = filtroMisFichas;
        notifyDataSetChanged();
    }

    @Override
    public void onBindViewHolder(FichaTestViewHolder holder, int position) {
        FichaTest fichaTest;
        if (filtroMisFichas) {
            fichaTest = fichasTestsCreador.get(position);
        } else {
            fichaTest = fichasTestsActivos.get(position);
        }
        if (fichaTest.isActivo()) {
            holder.textViewActivo.setText("Sí");
        } else {
            holder.textViewActivo.setText("No");
        }
        holder.textViewTitulo.setText(fichaTest.getTitulo());
        holder.textViewDescripcion.setText(fichaTest.getDescripcion());
        holder.textViewCreador.setText(fichaTest.getCreador());
        holder.textViewNumeroPreguntas.setText("" + fichaTest.getNumeroPreguntas());
        String ts = getDate(fichaTest.getFechaCreacion());
        holder.textViewFechaCreacion.setText(ts);
    }

    private String getDate(long time) {
        Calendar cal = Calendar.getInstance(Locale.ENGLISH);
        cal.setTimeInMillis(time);
        String date = DateFormat.format("dd-MM-yyyy, hh:mm:ss", cal).toString();
        return date;
    }

    @Override
    public int getItemCount() {
        if (filtroMisFichas) {
            if (fichasTestsCreador == null) return 0;
            else return fichasTestsCreador.size();
        } else {
            if (fichasTestsActivos == null) return 0;
            else return fichasTestsActivos.size();
        }
    }

    public void setOnClickListener(View.OnClickListener onClickListener) {
        this.onClickListener = onClickListener;
    }

    public void setOnLongClickListener(View.OnLongClickListener onLongClickListener) {
        this.onLongClickListener = onLongClickListener;
    }

    public static class FichaTestViewHolder extends RecyclerView.ViewHolder {
        TextView textViewActivo;
        TextView textViewTitulo;
        TextView textViewDescripcion;
        TextView textViewCreador;
        TextView textViewNumeroPreguntas;
        TextView textViewFechaCreacion;

        public FichaTestViewHolder(View itemView) {
            super(itemView);
            textViewActivo = (TextView) itemView.findViewById(R.id.textViewActivo);
            textViewTitulo = (TextView) itemView.findViewById(R.id.textViewTitulo);
            textViewDescripcion = (TextView) itemView.findViewById(R.id.textViewDescripcion);
            textViewNumeroPreguntas = (TextView) itemView.findViewById(R.id.textViewNumeroPreguntas);
            textViewCreador = (TextView) itemView.findViewById(R.id.textViewCreador);
            textViewFechaCreacion = (TextView) itemView.findViewById(R.id.textViewFechaCreacion);
        }
    }
}
