package com.raulvibo.tester;

import android.Manifest;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.StrictMode;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Toast;

import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.InstallCallbackInterface;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.core.Mat;
import org.opencv.core.Scalar;
import org.opencv.imgcodecs.Imgcodecs;
import org.opencv.imgproc.Imgproc;

import java.io.File;
import java.text.Normalizer;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import static android.support.design.widget.Snackbar.make;

public class CamaraActivity extends AppCompatActivity implements
        CameraBridgeViewBase.CvCameraViewListener2,
        LoaderCallbackInterface {
    private static final int SOLICITUD_INSTALACION_OPENCV_MANAGER = 0;
    private static final int SOLICITUD_PERMISO_CAMARA = 1;
    private static final int SOLICITUD_PERMISO_ALMACENAMIENTO = 3;
    private static final int ACCION_CARGAR_OPENCV = 4;
    private CameraBridgeViewBase cameraView;
    private View vista;
    private Mat entrada;
    private Mat entrada_gris;
    private Mat salida;
    private Mat ultimaSalidaEstable;
    Procesador procesador;
    int estado;
    boolean lecturaEstable;
    Test test;
    Test ultimoTestValido;
    String titulo;
    String solucion;
    double puntuacionMaxima;
    Scalar colorSuspenso;
    Scalar colorAprobado;
    FloatingActionButton fabCancelar;
    FloatingActionButton fabRefrescar;
    FloatingActionButton fabGuardarImagen;
    Snackbar snackbarGuardar;
    Snackbar snackbarCorregir;
    Snackbar snackbarActualizar;
    Snackbar snackbarGuardarImagen;
    static final String TAG = "CamaraActivity";
    String accion;
    String key;
    String photoPathOut;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.activity_camara);
        StrictMode.VmPolicy.Builder builder = new StrictMode.VmPolicy.Builder();
        StrictMode.setVmPolicy(builder.build());
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        cameraView = (CameraBridgeViewBase) findViewById(R.id.vista_camara);
        cameraView.setCvCameraViewListener(this);
        cameraView.disableFpsMeter();
        vista = findViewById(R.id.content_main);
        test = new Test();
        puntuacionMaxima = 10;
        colorSuspenso = new Scalar(204, 0, 0);
        colorAprobado = new Scalar(102, 153, 0);
        fabCancelar = (FloatingActionButton) findViewById(R.id.fabCancelar);
        fabCancelar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                finish();
            }
        });
        fabRefrescar = (FloatingActionButton) findViewById(R.id.fabRefrescar);
        fabRefrescar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                snackbarGuardar.dismiss();
                snackbarCorregir.dismiss();
                snackbarActualizar.dismiss();
                snackbarGuardarImagen.dismiss();
                fabRefrescar.setVisibility(View.INVISIBLE);
                fabGuardarImagen.setVisibility(View.INVISIBLE);
                estado = 0;
                cameraView.enableView();
            }
        });
        fabGuardarImagen = (FloatingActionButton) findViewById(R.id.fabGuardarImagen);
        fabGuardarImagen.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                pedirPermisosAlmacenamiento();
            }
        });
        snackbarGuardar = Snackbar.make(fabRefrescar, "Test escaneado", Snackbar.LENGTH_INDEFINITE)
                .setAction("Guardar", new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        guardarSolucion();
                    }
                })
                .setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbarGuardar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbarCorregir = Snackbar.make(fabRefrescar, "Test escaneado", Snackbar.LENGTH_INDEFINITE)
                .setAction("Corregir", new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        corregir();

                    }
                }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbarCorregir.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbarActualizar = Snackbar.make(fabRefrescar, "Test escaneado", Snackbar.LENGTH_INDEFINITE)
                .setAction("Actualizar", new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        actualizar();
                    }
                }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbarActualizar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbarGuardarImagen = Snackbar.make(fabRefrescar, "Imagen guardada", Snackbar.LENGTH_INDEFINITE)
                .setAction("Ver", new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Intent intent = new Intent();
                        intent.setAction(android.content.Intent.ACTION_VIEW);
                        intent.setDataAndType(Uri.fromFile(new File(photoPathOut)), "image/png");
                        intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                        startActivity(intent);
                    }
                }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbarGuardarImagen.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        gestionarAccion();
    }

    private void gestionarAccion() {
        accion = getIntent().getExtras().getString("accion");
        switch (accion) {
            case "corregir":
                accionCorregir();
                break;
            case "actualizar":
                accionActualizar();
                break;
            default:
                break;
        }
    }

    private void accionCorregir() {
        // https://www.drillio.com/en/2011/java-remove-accent-diacritic/
        titulo = getIntent().getExtras().getString("titulo");
        titulo = Normalizer.normalize(titulo, Normalizer.Form.NFD).replaceAll("\\p{InCombiningDiacriticalMarks}+", "");
        ;
        solucion = getIntent().getExtras().getString("solucion");
    }

    private void accionActualizar() {
        key = getIntent().getExtras().getString("key");
    }

    // https://stackoverflow.com/questions/11392183/how-to-check-programmatically-if-an-application-is-installed-or-not-in-android
    public static boolean isPackageInstalled(Context context, String packageName) {
        final PackageManager packageManager = context.getPackageManager();
        Intent intent = packageManager.getLaunchIntentForPackage(packageName);
        if (intent == null) {
            return false;
        }
        List<ResolveInfo> list = packageManager.queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY);
        return !list.isEmpty();
    }

    // https://stackoverflow.com/questions/11753000/how-to-open-the-google-play-store-directly-from-my-android-application
    public void instalarApp(String appPackageName) {
        try {
            startActivityForResult(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + appPackageName)), SOLICITUD_INSTALACION_OPENCV_MANAGER);
        } catch (android.content.ActivityNotFoundException anfe) {
            startActivityForResult(new Intent(Intent.ACTION_VIEW, Uri.parse("https://play.google.com/store/apps/details?id=" + appPackageName)), SOLICITUD_INSTALACION_OPENCV_MANAGER);
        }
    }

    public void comprobarOpenCVManagerInstalado() {
        if (isPackageInstalled(this, "org.opencv.engine")) {
            ((Aplicacion) getApplication()).setOpencvManagerInstalado(true);
            cargarOpenCV();
        } else {
            ((Aplicacion) getApplication()).setOpencvManagerInstalado(false);
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setMessage("Se necesita instalar la aplicación OpenCV Manager.")
                    .setPositiveButton("De acuerdo", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            instalarApp("org.opencv.engine");
                        }
                    })
                    .setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            finish();
                        }
                    });
            builder.create().show();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case SOLICITUD_INSTALACION_OPENCV_MANAGER:
                if (isPackageInstalled(this, "org.opencv.engine")) {
                    ((Aplicacion) getApplication()).setOpencvManagerInstalado(true);
                    cargarOpenCV();
                } else {
                    ((Aplicacion) getApplication()).setOpencvManagerInstalado(false);
                    finish();
                }
                break;
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        fabRefrescar.setVisibility(View.INVISIBLE);
        fabGuardarImagen.setVisibility(View.INVISIBLE);
        comprobarPermisos(ACCION_CARGAR_OPENCV);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    void comprobarPermisos(int accion) {
        switch (accion) {
            case ACCION_CARGAR_OPENCV:
                comprobarOpenCVManagerInstalado();
                pedirPermisosCamara();
                break;
        }
    }

    void cargarOpenCV() {
        if (((Aplicacion) getApplication()).sePuedeCargarOpenCV())
            OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_3_0_0, this, this);
    }

    @Override
    public void onPause() {
        if (cameraView != null)
            cameraView.disableView();
        if (getIntent().hasExtra("solucion"))
            getIntent().getExtras().remove("solucion");
        snackbarGuardar.dismiss();
        snackbarCorregir.dismiss();
        snackbarActualizar.dismiss();
        snackbarGuardarImagen.dismiss();
        super.onPause();
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        if (cameraView != null)
            cameraView.disableView();
    }

    public void pedirPermisosCamara() {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            ((Aplicacion) getApplication()).setPermisoCamara(false);
            if (ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.CAMERA)) {
                Snackbar snackbar = make(vista, "Se necesita permiso para usar la cámara.", Snackbar.LENGTH_INDEFINITE).setAction("OK", new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        ActivityCompat.requestPermissions(CamaraActivity.this, new String[]{Manifest.permission.CAMERA}, SOLICITUD_PERMISO_CAMARA);
                    }
                });
                snackbar.show();
            } else {
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.CAMERA}, SOLICITUD_PERMISO_CAMARA);
            }
        } else {
            ((Aplicacion) getApplication()).setPermisoCamara(true);
            cargarOpenCV();
        }
    }


    @Override
    public void onManagerConnected(int status) {
        switch (status) {
            case LoaderCallbackInterface.SUCCESS:
                estado = 0;
                lecturaEstable = false;
                cameraView.enableView();
                entrada_gris = new Mat();
                ultimaSalidaEstable = new Mat();
                /*
                Camera camera = open(0);
                Camera.Parameters params = camera.getParameters();
                List<String> FocusModes = params.getSupportedFocusModes();
                for (int i = 0; i < FocusModes.size(); i++) {
                    Log.d("Camera", "" + i + "=" + FocusModes.get(i).toString());
                }
                if (FocusModes.contains(Camera.Parameters.FOCUS_MODE_AUTO)) {
                    params.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
                    camera.setParameters(params);
                }
                */
                break;
            default:
                Toast.makeText(CamaraActivity.this, "OpenCV no se cargó correctamente. En algunos casos, esta incidencia se ha resuelto desinstalando la aplicación OpenCV Manager y volviéndola a instalar desde el Play Store", Toast.LENGTH_LONG).show();
                finish();
                break;
        }
    }

    @Override
    public void onPackageInstall(int operation, InstallCallbackInterface callback) {

    }

    @Override
    public void onCameraViewStarted(int anchura, int altura) {
        salida = new Mat();
        procesador = new Procesador();
    }

    @Override
    public void onCameraViewStopped() {}

    @Override
    public Mat onCameraFrame(CameraBridgeViewBase.CvCameraViewFrame inputFrame) {
        switch (estado) {
            case 0: // Lectura de la cámara
                entrada = inputFrame.rgba();
                entrada_gris = inputFrame.gray();
                test = new Test();
                lecturaEstable = procesador.procesa(entrada_gris, salida, test);
                entrada.copyTo(salida);
                if (lecturaEstable) {
                    salida.copyTo(ultimaSalidaEstable);
                    ultimoTestValido = test;
                    procesador.dibujarCasillas(salida, test);
                    estado = 1;
                    lanzarMensaje();
                }
                break;
            case 1: // Muestra test escaneado
                ultimaSalidaEstable.copyTo(salida);
                procesador.dibujarCasillas(salida, ultimoTestValido);
                break;
            case 3: // Muestra el resultado de la corrección
                if (ultimoTestValido.mismaEstructura(solucion)) {
                    String correccion = ultimoTestValido.getCorreccion(solucion);
                    procesador.dibujarCorreccion(ultimaSalidaEstable, ultimoTestValido, correccion);
                    ultimaSalidaEstable.copyTo(salida);
                    double puntuacion = ultimoTestValido.getPuntuacion(puntuacionMaxima, correccion, false);
                    String usuario = ((Aplicacion) getApplication()).getUsername();
                    String fecha = getDate(System.currentTimeMillis());
                    procesador.mostrarTitulo(ultimaSalidaEstable, titulo, usuario, fecha);
                    procesador.mostrarPuntuacion(ultimaSalidaEstable, puntuacion, puntuacionMaxima);
                    ultimaSalidaEstable.copyTo(salida);
                    estado = 4;
                }
                break;
            case 4: // Una vez mostrada la corrección, dejamos de actualizar la pantalla.
                ultimaSalidaEstable.copyTo(salida);
                desactivarCameraView();
                break;
        }
        return salida;
    }

    void desactivarCameraView() {
        CamaraActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                cameraView.disableView();
            }
        });
    }

    private boolean takePhoto(final Mat output) {
        // Determina la ruta para crear los archivos
        final long currentTimeMillis = System.currentTimeMillis();
        final String appName = getString(R.string.app_name);
        final String galleryPath = Environment
                .getExternalStoragePublicDirectory(
                        Environment.DIRECTORY_PICTURES).toString();
        final String albumPath = galleryPath + "/" + appName;
        photoPathOut = albumPath + "/Out_" + currentTimeMillis + ".png";
        boolean resultado = true;
        // Asegurarse que el directorio existe
        File album = new File(albumPath);
        if (!album.isDirectory() && !album.mkdirs()) {
            Log.e(TAG, "Error al crear el directorio " + albumPath);
            return false;
        }
        // Intenta crear los archivos
        Mat mBgr = new Mat();
        if (output.channels() == 1)
            Imgproc.cvtColor(output, mBgr, Imgproc.COLOR_GRAY2BGR, 3);
        else
            Imgproc.cvtColor(output, mBgr, Imgproc.COLOR_RGBA2BGR, 3);
        if (!Imgcodecs.imwrite(photoPathOut, mBgr)) {
            Log.e(TAG, "Fallo al guardar " + photoPathOut);
            resultado = false;
        }
        mBgr.release();
        return resultado;
    }

    public void pedirPermisosAlmacenamiento() {
        vista = findViewById(R.id.content_main);
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            if (ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                make(vista, "Se necesita permiso para usar el almacenamiento.", Snackbar.LENGTH_INDEFINITE)
                        .setAction("OK", new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
                                ActivityCompat.requestPermissions(CamaraActivity.this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, SOLICITUD_PERMISO_ALMACENAMIENTO);
                            }
                        }).show();
            } else {
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, SOLICITUD_PERMISO_ALMACENAMIENTO);
            }
        } else {
            guardarImagen();
        }
    }

    void guardarImagen() {
        if (takePhoto(salida)) {
            snackbarGuardarImagen.show();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case SOLICITUD_PERMISO_CAMARA: {
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    ((Aplicacion) getApplication()).setPermisoCamara(true);
                    cargarOpenCV();
                } else {
                    Snackbar snackbar = make(vista, "Al no tener permiso para usar la cámara queda deshabilitada esta funcionalidad", Snackbar.LENGTH_INDEFINITE)
                            .setAction("OK", new View.OnClickListener() {
                                @Override
                                public void onClick(View view) {
                                    finish();
                                }
                            }).setActionTextColor(getResources().getColor(R.color.colorAccent));
                    snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                    snackbar.show();
                }
                break;
            }
            case SOLICITUD_PERMISO_ALMACENAMIENTO: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    takePhoto(salida);
                    mostrarSnackbar("Imagen guardada");
                } else {
                    Snackbar snackbar = make(vista, "Al no tener permiso de almacenamiento no se puede guardar la imagen.", Snackbar.LENGTH_INDEFINITE)
                            .setAction("OK", new View.OnClickListener() {
                                @Override
                                public void onClick(View view) {
                                }
                            }).setActionTextColor(getResources().getColor(R.color.colorAccent));
                    snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                    snackbar.show();
                }
                break;
            }
        }
    }

    public void guardarSolucion() {
        Intent intent = new Intent(CamaraActivity.this, FichaTestActivity.class);
        intent.putExtra("accion", "guardar");
        intent.putExtra("solucion", ultimoTestValido.getCodificacion());
        intent.putExtra("numeroPreguntas", ultimoTestValido.getNumeroPreguntas());
        intent.putExtra("fechaCreacion", System.currentTimeMillis());
        startActivity(intent);
    }

    public void corregir() {
        estado = 3;
        cameraView.enableView();
        fabGuardarImagen.setVisibility(View.VISIBLE);
    }

    public void actualizar() {
        Intent intent = new Intent(CamaraActivity.this, FichaTestActivity.class);
        intent.putExtra("accion", "actualizar");
        intent.putExtra("key", key);
        intent.putExtra("solucion", ultimoTestValido.getCodificacion());
        intent.putExtra("numeroPreguntas", ultimoTestValido.getNumeroPreguntas());
        startActivity(intent);
    }

    public void lanzarMensaje() {
        CamaraActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                switch (accion) {
                    case "crear":
                        cameraView.disableView();
                        fabRefrescar.setVisibility(View.VISIBLE);
                        snackbarGuardar.show();
                        break;
                    case "corregir":
                        if (ultimoTestValido.mismaEstructura(solucion)) {
                            cameraView.disableView();
                            fabRefrescar.setVisibility(View.VISIBLE);
                            snackbarCorregir.show();
                        } else {
                            estado = 0;
                        }
                        break;
                    case "actualizar":
                        cameraView.disableView();
                        fabRefrescar.setVisibility(View.VISIBLE);
                        snackbarActualizar.show();
                        break;
                }
            }
        });
    }

    void mostrarSnackbar(String mensaje) {
        Snackbar snackbar = make(vista, mensaje, Snackbar.LENGTH_LONG)
                .setAction("OK", new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                    }
                }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbar.show();
    }

    private String getDate(long time) {
        Calendar cal = Calendar.getInstance(Locale.ENGLISH);
        cal.setTimeInMillis(time);
        String date = DateFormat.format("dd-MM-yyyy, hh:mm:ss", cal).toString();
        return date;
    }
}
