package com.raulvibo.tester;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.format.DateFormat;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * Created by raulvibo on 03/09/2017.
 */

public class FichaTestActivity extends AppCompatActivity {
    Switch switchActivo;
    EditText editTextTitulo;
    EditText editTextDescripcion;
    TextView textViewNumeroPreguntas;
    TextView textViewCreador;
    TextView textViewFechaCreacion;
    String key = null;
    int numeroPreguntas;
    long fechaCreacion;
    String solucion;
    View vista;
    String accion;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_ficha_test);
        vista = findViewById(R.id.content_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        switchActivo = (Switch) findViewById(R.id.switchActivo);
        editTextTitulo = (EditText) findViewById(R.id.editTextTitulo);
        editTextDescripcion = (EditText) findViewById(R.id.editTextDescripcion);
        textViewNumeroPreguntas = (TextView) findViewById(R.id.textViewNumeroPreguntas);
        textViewCreador = (TextView) findViewById(R.id.textViewCreador);
        textViewFechaCreacion = (TextView) findViewById(R.id.textViewFechaCreacion);
        gestionarAccion();
    }

    private void gestionarAccion() {
        accion = getIntent().getExtras().getString("accion");
        switch (accion) {
            case "editar":
                accionEditar();
                break;
            case "guardar":
                accionGuardar();
                break;
            case "actualizar":
                accionActualizar();
                break;
        }
    }

    private void accionEditar() {
        key = getIntent().getExtras().getString("key");
        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference databaseReference = database.getReference("tests/" + key);
        databaseReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                // This method is called once with the initial value and again
                // whenever data at this location is updated.
                FichaTest fichaTest = dataSnapshot.getValue(FichaTest.class);
                if (fichaTest != null) {
                    solucion = fichaTest.getSolucion();
                    switchActivo.setChecked(fichaTest.isActivo());
                    editTextTitulo.setText(fichaTest.getTitulo());
                    editTextDescripcion.setText(fichaTest.getDescripcion());
                    textViewNumeroPreguntas.setText("" + fichaTest.getNumeroPreguntas());
                    textViewCreador.setText(fichaTest.getCreador());
                    textViewFechaCreacion.setText("" + getDate(fichaTest.getFechaCreacion()));
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
            }
        });
    }

    @Override
    protected void onSaveInstanceState(Bundle bundle) {
        bundle.putString("accion", "actualizar");
        bundle.putString("titulo", editTextTitulo.getText().toString());
        bundle.putString("descripcion", editTextDescripcion.getText().toString());
        bundle.putString("fechaCreacion", textViewFechaCreacion.getText().toString());
        super.onSaveInstanceState(bundle);
    }

    @Override
    protected void onRestoreInstanceState(Bundle bundle) {
        super.onRestoreInstanceState(bundle);
        if (bundle != null) {
            editTextTitulo.setText(bundle.getString("titulo"));
            editTextDescripcion.setText(bundle.getString("descripcion"));
            textViewFechaCreacion.setText(bundle.getString("fechaCreacion"));
        }
    }

    private void accionGuardar() {
        Bundle extras = getIntent().getExtras();
        solucion = extras.getString("solucion");
        numeroPreguntas = extras.getInt("numeroPreguntas");
        fechaCreacion = extras.getLong("fechaCreacion");
        switchActivo.setChecked(false);
        editTextTitulo.setText("");
        editTextDescripcion.setText("");
        textViewNumeroPreguntas.setText("" + numeroPreguntas);
        textViewCreador.setText(((Aplicacion) getApplication()).getUsuario());
        textViewFechaCreacion.setText("" + getDate(fechaCreacion));
    }

    private void accionActualizar() {
        key = getIntent().getExtras().getString("key");
        solucion = getIntent().getExtras().getString("solucion");
        numeroPreguntas = getIntent().getExtras().getInt("numeroPreguntas");
        textViewNumeroPreguntas.setText("" + numeroPreguntas);
        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference databaseReference = database.getReference("tests/" + key);
        databaseReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                // This method is called once with the initial value and again
                // whenever data at this location is updated.
                FichaTest fichaTest = dataSnapshot.getValue(FichaTest.class);
                if (fichaTest != null) {
                    switchActivo.setChecked(fichaTest.isActivo());
                    editTextTitulo.setText(fichaTest.getTitulo());
                    editTextDescripcion.setText(fichaTest.getDescripcion());
                    textViewCreador.setText(fichaTest.getCreador());
                    textViewFechaCreacion.setText("" + getDate(fichaTest.getFechaCreacion()));
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
            }
        });

    }

    private String getDate(long time) {
        Calendar cal = Calendar.getInstance(Locale.ENGLISH);
        cal.setTimeInMillis(time);
        String date = DateFormat.format("dd-MM-yyyy, hh:mm:ss", cal).toString();
        return date;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_ficha_test, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.guardar:
                guardar();
                break;
            case R.id.cancelar:
                finish();
                break;
        }
        return true;
    }

    void lanzarListado() {
        Intent intent = new Intent(this, ListadoFichaTestActivity.class);
        startActivity(intent);
    }

    void guardar() {
        if (!revisaFormulario()) return;
        if (key == null) key = guardaFichaTest();
        else actualizarFichaTest();
        hideVirtualKeyboard();
        Snackbar snackbar = Snackbar.make(vista, "Test guardado", Snackbar.LENGTH_INDEFINITE).setAction("OK", new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                lanzarListado();
            }
        }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbar.show();
    }

    // https://stackoverflow.com/questions/1109022/close-hide-the-android-soft-keyboard
    void hideVirtualKeyboard() {
        // Check if no view has focus:
        View view = this.getCurrentFocus();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

    String guardaFichaTest() {
        FichaTest fichaTest = new FichaTest.FichaTestBuilder()
                .isActivo(switchActivo.isChecked())
                .withTitulo(editTextTitulo.getText().toString())
                .withDescripcion(editTextDescripcion.getText().toString())
                .withCreador(textViewCreador.getText().toString())
                .withNumeroPreguntas(numeroPreguntas)
                .withFechaCreacion(fechaCreacion)
                .withSolucion(solucion)
                .build();
        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference databaseReference = database.getReference("tests");
        DatabaseReference newPostRef = databaseReference.push();
        newPostRef.setValue(fichaTest);
        return newPostRef.getKey();
    }

    void actualizarFichaTest() {
        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference databaseReference = database.getReference("tests/" + key);
        Map<String, Object> taskMap = new HashMap<>();
        taskMap.put("activo", switchActivo.isChecked());
        taskMap.put("titulo", editTextTitulo.getText().toString());
        taskMap.put("descripcion", editTextDescripcion.getText().toString());
        taskMap.put("numeroPreguntas", Integer.parseInt(textViewNumeroPreguntas.getText().toString()));
        taskMap.put("solucion", solucion);
        databaseReference.updateChildren(taskMap);
    }

    boolean revisaFormulario() {
        boolean revision = true;
        if (isEmpty(editTextTitulo)) {
            Toast.makeText(FichaTestActivity.this, "Es necesario incluir un título.", Toast.LENGTH_LONG).show();
            revision = false;
        }
        if (isEmpty(editTextDescripcion)) {
            Toast.makeText(FichaTestActivity.this, "Es necesario incluir una descripción.", Toast.LENGTH_LONG).show();
            revision = false;
        }
        return revision;
    }

    private boolean isEmpty(EditText etText) {
        return etText.getText().toString().trim().length() == 0;
    }
}
