package com.raulvibo.tester;

import android.Manifest;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TabLayout;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static android.support.design.widget.Snackbar.make;


/**
 * Created by raulvibo on 30/08/2017.
 */

public class ListadoFichaTestActivity extends AppCompatActivity {
    RecyclerView mRecyclerView;
    RecyclerView.LayoutManager mLayoutManager;
    AdaptadorFichaTest adaptadorFichaTest;
    FirebaseDatabase database;
    List<FichaTest> fichasTestsActivos;
    List<String> keysFichasTestsActivos;
    List<FichaTest> fichasTestsCreador;
    List<String> keysFichasTestsCreador;
    TabLayout tabLayout;
    View vista;
    ActionMode actionMode;
    private static final int SOLICITUD_PERMISO_OBTENER_CUENTAS = 2;
    Menu menu;
    ProgressBar progressBar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        fichasTestsActivos = ((Aplicacion) getApplication()).getFichasTestsActivos();
        keysFichasTestsActivos = ((Aplicacion) getApplication()).getKeysFichasTestsActivos();
        fichasTestsCreador = ((Aplicacion) getApplication()).getFichasTestsCreador();
        keysFichasTestsCreador = ((Aplicacion) getApplication()).getKeysFichasTestsCreador();
        setContentView(R.layout.activity_listado);
        vista = findViewById(R.id.content_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        tabLayout = (TabLayout) findViewById(R.id.tab_layout);
        tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                if (actionMode != null) actionMode.finish();
                switch (tab.getPosition()) {
                    case 0:
                        menu.findItem(R.id.crear).setEnabled(false);
                        menu.findItem(R.id.crear).setVisible(false);
                        adaptadorFichaTest.setFiltroMisFichas(false);
                        break;
                    case 1:
                        menu.findItem(R.id.crear).setEnabled(true);
                        menu.findItem(R.id.crear).setVisible(true);
                        adaptadorFichaTest.setFiltroMisFichas(true);
                        break;
                }
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {

            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {

            }
        });

        mRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_listado);
        mRecyclerView.setHasFixedSize(false);

        // use a linear layout manager
        mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);

        adaptadorFichaTest = new AdaptadorFichaTest(fichasTestsActivos, fichasTestsCreador);

        adaptadorFichaTest.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(ListadoFichaTestActivity.this, CamaraActivity.class);
                String solucion = getFichaTest(v).getSolucion();
                String titulo = getFichaTest(v).getTitulo();
                i.putExtra("accion", "corregir");
                i.putExtra("titulo", titulo);
                i.putExtra("solucion", solucion);
                startActivity(i);
            }
        });

        adaptadorFichaTest.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(final View v) {
                if (tabLayout.getSelectedTabPosition() == 1)
                    actionMode = startActionMode(new ActionMode.Callback() {
                        @Override
                        public boolean onCreateActionMode(ActionMode mode, Menu menu) {
                            mode.setTitle("Test seleccionado");
                            MenuInflater inflater = mode.getMenuInflater();
                            inflater.inflate(R.menu.menu_contextual, menu);
                            v.setBackground(getResources().getDrawable(R.drawable.border_highlight));
                            return true;
                        }

                        @Override
                        public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
                            return false;
                        }

                        @Override
                        public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
                            switch (item.getItemId()) {
                                case R.id.activar:
                                    String key = getKeyFichaTest(v);
                                    if (getFichaTest(v).isActivo()){
                                        cambiarActivacionFichaTest(key, !getFichaTest(v).isActivo());
                                        mostrarSnackbar("Test no activado");
                                    }
                                    else {
                                        cambiarActivacionFichaTest(key, !getFichaTest(v).isActivo());
                                        mostrarSnackbar("Test activado");
                                    }
                                    mode.finish();
                                    break;
                                case R.id.editar:
                                    Intent i = new Intent(ListadoFichaTestActivity.this, FichaTestActivity.class);
                                    i.putExtra("accion", "editar");
                                    key = getKeyFichaTest(v);
                                    i.putExtra("key", key);
                                    startActivity(i);
                                    mode.finish();
                                    break;
                                case R.id.actualizar:
                                    Intent intent = new Intent(ListadoFichaTestActivity.this, CamaraActivity.class);
                                    intent.putExtra("accion", "actualizar");
                                    key = getKeyFichaTest(v);
                                    intent.putExtra("key", key);
                                    startActivity(intent);
                                    break;
                                case R.id.eliminar:
                                    key = getKeyFichaTest(v);
                                    confirmarEliminacionTest(key);
                                    mode.finish();
                                    break;
                                default:
                                    return false;
                            }
                            return true;
                        }

                        @Override
                        public void onDestroyActionMode(ActionMode mode) {
                            v.setBackground(getResources().getDrawable(R.drawable.border));
                        }
                    });
                return true;
            }
        });
        mRecyclerView.setAdapter(adaptadorFichaTest);
        progressBar = (ProgressBar) findViewById(R.id.progressBar);
    }

    @Override
    public void onStart() {
        super.onStart();
        preLeerBBDD(Manifest.permission.GET_ACCOUNTS, SOLICITUD_PERMISO_OBTENER_CUENTAS);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    void eliminarFichaTest(String key) {
        database = FirebaseDatabase.getInstance();
        database.getReference("tests/" + key).removeValue();
    }

    void cambiarActivacionFichaTest(String key, boolean activo) {
        FirebaseDatabase database = FirebaseDatabase.getInstance();
        DatabaseReference databaseReference = database.getReference("tests/" + key);
        Map<String, Object> taskMap = new HashMap<>();
        taskMap.put("activo", activo);
        databaseReference.updateChildren(taskMap);
    }

    FichaTest getFichaTest(View v) {
        FichaTest fichaTest;
        int indice = mRecyclerView.getChildAdapterPosition(v);
        if (adaptadorFichaTest.isFiltroMisFichas()) {
            fichaTest = ((Aplicacion) getApplication()).getFichasTestsCreador().get(indice);

        } else {
            fichaTest = ((Aplicacion) getApplication()).getFichasTestsActivos().get(indice);
        }
        return fichaTest;
    }

    String getKeyFichaTest(View v) {
        String key;
        int indice = mRecyclerView.getChildAdapterPosition(v);
        if (adaptadorFichaTest.isFiltroMisFichas()) {
            key = ((Aplicacion) getApplication()).getKeysFichasTestsCreador().get(indice);

        } else {
            key = ((Aplicacion) getApplication()).getKeysFichasTestsActivos().get(indice);
        }
        return key;
    }

    void leerBBDD() {
        // Rellenamos la lista de fichasTestsActivos
        database = FirebaseDatabase.getInstance();
        database.getReference("tests").orderByChild("activo").equalTo(true)
                .addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        progressBar.setVisibility(View.VISIBLE);
                        fichasTestsActivos.removeAll(fichasTestsActivos);
                        keysFichasTestsActivos.removeAll(keysFichasTestsActivos);
                        for (DataSnapshot childDataSnapshot : dataSnapshot.getChildren()) {
                            FichaTest fichaTest = childDataSnapshot.getValue(FichaTest.class);
                            fichasTestsActivos.add(fichaTest);
                            String keyFichaTest = childDataSnapshot.getKey();
                            keysFichasTestsActivos.add(keyFichaTest);
                        }
                        adaptadorFichaTest.notifyDataSetChanged();
                        progressBar.setVisibility(View.INVISIBLE);
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {}
                });
        database.getReference("tests").orderByChild("creador").equalTo(((Aplicacion) getApplication()).getUsuario())
                .addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        fichasTestsCreador.removeAll(fichasTestsCreador);
                        keysFichasTestsCreador.removeAll(keysFichasTestsCreador);
                        for (DataSnapshot childDataSnapshot : dataSnapshot.getChildren()) {
                            FichaTest fichaTest = childDataSnapshot.getValue(FichaTest.class);
                            fichasTestsCreador.add(fichaTest);
                            String keyFichaTest = childDataSnapshot.getKey();
                            keysFichasTestsCreador.add(keyFichaTest);
                        }
                        adaptadorFichaTest.notifyDataSetChanged();
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {}
                });

    }

    public void preLeerBBDD(final String permiso, final int solicitud) {
        if (ContextCompat.checkSelfPermission(this, permiso) == PackageManager.PERMISSION_GRANTED) {
            ((Aplicacion) getApplication()).setPermisoObtenerCuentas(true);
            leerBBDD();
        }
        else {
            ((Aplicacion) getApplication()).setPermisoObtenerCuentas(false);
            if (ActivityCompat.shouldShowRequestPermissionRationale(this, permiso)) {
                Snackbar snackbar = make(vista, "Se necesita permiso para obtener las cuentas", Snackbar.LENGTH_INDEFINITE)
                        .setAction("OK", new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
                                ActivityCompat.requestPermissions(ListadoFichaTestActivity.this, new String[]{permiso}, solicitud);
                            }
                        }).setActionTextColor(getResources().getColor(R.color.colorAccent));
                snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                snackbar.show();
            } else {
                ActivityCompat.requestPermissions(ListadoFichaTestActivity.this, new String[]{permiso}, solicitud);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case SOLICITUD_PERMISO_OBTENER_CUENTAS: {
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    leerBBDD();
                } else {
                    leerBBDD();
                }
                break;
            }
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_listado, menu);
        this.menu = menu;
        menu.findItem(R.id.crear).setEnabled(false);
        menu.findItem(R.id.crear).setVisible(false);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.crear:
                lanzarCamaraActivity();
                break;
        }
        return true;
    }

    void lanzarCamaraActivity() {
        Intent i = new Intent(ListadoFichaTestActivity.this, CamaraActivity.class);
        i.putExtra("accion", "crear");
        startActivity(i);
    }

    //https://stackoverflow.com/questions/11740311/android-confirmation-message-for-delete
    private void confirmarEliminacionTest(final String key) {
        AlertDialog alertDialog = new AlertDialog.Builder(this)
                .setTitle("Eliminar")
                .setMessage("¿Quieres eliminar este test?")
                .setIcon(R.drawable.ic_remove_circle)
                .setPositiveButton("Sí", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        //your deleting code
                        eliminarFichaTest(key);
                        mostrarSnackbar("Test eliminado");
                        dialog.dismiss();
                    }
                })
                .setNegativeButton("No", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.dismiss();
                    }
                })
                .create();
        alertDialog.show();
    }

    void mostrarSnackbar(String mensaje){
        Snackbar snackbar = make(vista, mensaje, Snackbar.LENGTH_LONG)
                .setAction("OK", new View.OnClickListener() {
            @Override
            public void onClick(View view) {}
        }).setActionTextColor(getResources().getColor(R.color.colorAccent));
        snackbar.getView().setBackgroundColor(getResources().getColor(R.color.colorPrimary));
        snackbar.show();
    }
}
