package com.raulvibo.tester;

import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Rect;
import org.opencv.imgproc.Imgproc;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by raulvibo on 21/08/2017.
 */

public class Test implements Cloneable {
    List<Pregunta> preguntas;

    public Test() {
        preguntas = new ArrayList<>();
    }

    public List<Pregunta> getPreguntas() {
        return preguntas;
    }

    public void setPreguntas(List<Pregunta> preguntas) {
        this.preguntas = preguntas;
    }

    public void borrar() {
        preguntas.clear();
    }

    public void rellenar(Mat entrada, List<Rect> casillas) {
        borrar();
        int distanciaEntreCasillas = 0;
        boolean mismaLinia = true;
        boolean mismoGrupo = true;
        if (casillas != null && casillas.size() > 0) {
            Pregunta pregunta = new Pregunta();
            preguntas.add(pregunta);
            Opcion opcion, opcionAnterior = null;
            for (int c = 0; c < casillas.size(); c++) {
                Rect casilla = casillas.get(c);
                boolean valor = leerCasilla(entrada, casilla);
                opcion = new Opcion(valor, casilla);
                if (opcionAnterior != null) {
                    mismaLinia = opcion.mismaLinia(opcionAnterior);
                }
                if (opcionAnterior != null && c > 1) {
                    mismoGrupo = opcion.mismoGrupo(opcionAnterior, distanciaEntreCasillas);
                }
                if (mismaLinia && !mismoGrupo) {
                    pregunta = new Pregunta();
                    preguntas.add(pregunta);
                }
                if (!mismaLinia) {
                    pregunta = new Pregunta();
                    preguntas.add(pregunta);
                }
                pregunta.addOpcion(opcion);
                opcionAnterior = opcion;
                if (c == 1) {
                    if (pregunta.getNumeroOpciones() == 2) {
                        distanciaEntreCasillas = pregunta.getOpcion(0).distancia(pregunta.getOpcion(1));
                    } else {
                        preguntas.clear();
                        return;
                    }
                }
            }
        }
    }

    public boolean leerCasilla(Mat entrada, Rect casilla) {
        Rect rectCorregida = new Rect();
        reducirBB(casilla, rectCorregida);
        Mat recorte = entrada.submat(rectCorregida);
        List<MatOfPoint> contorno = new ArrayList<MatOfPoint>();
        Mat hierarchy = new Mat();
        Imgproc.threshold(recorte, recorte, 0, 255, Imgproc.THRESH_OTSU);
        Imgproc.findContours(recorte, contorno, hierarchy, Imgproc.RETR_LIST, Imgproc.CHAIN_APPROX_SIMPLE);
        return (contorno.size() >= 3);
    }


    void reducirBB(Rect entrada, Rect salida) {
        double margen = 0.20;
        int margenHorizontal = (int) (entrada.width * margen);
        int margenVertical = (int) (entrada.height * margen);
        salida.x = entrada.x + margenHorizontal;
        salida.y = entrada.y + margenVertical;
        salida.width = entrada.width - 2 * margenHorizontal;
        salida.height = entrada.height - 2 * margenVertical;
    }


    public int getNumeroPreguntas() {
        return preguntas.size();
    }

    public Pregunta getPregunta(int p) {
        return preguntas.get(p);
    }

    public String getCodificacion() {
        String codificacion = "";
        for (int p = 0; p < getNumeroPreguntas(); p++) {
            Pregunta pregunta = getPregunta(p);
            for (int r = 0; r < pregunta.getNumeroOpciones(); r++) {
                Opcion opcion = pregunta.getOpcion(r);
                boolean valor = opcion.getValor();
                if (valor) codificacion = codificacion + 'T';
                else codificacion = codificacion + 'F';
            }
            codificacion = codificacion + '*';
        }
        return codificacion;
    }

    public String getCorreccion(String solucion) {
        String codificacion = getCodificacion();
        String correccion = "";
        if (mismaEstructura(solucion)) {
            for (int i = 0; i < codificacion.length(); i++) {
                switch (codificacion.charAt(i)) {
                    case 'T':
                        correccion = correccion + solucion.charAt(i);
                        break;
                    case 'F':
                        correccion = correccion + ("" + solucion.charAt(i)).toLowerCase();
                        break;
                    case '*':
                        correccion = correccion + '*';
                }
            }
        }
        return correccion;
    }

    public boolean mismaEstructura(String solucion) {
        String codificacion = getCodificacion();
        boolean resultado = (codificacion.length() == solucion.length());
        if (resultado) {
            for (int i = 0; i < codificacion.length(); i++) {
                if (codificacion.charAt(i) == '*' || solucion.charAt(i) == '*')
                    resultado = resultado && (codificacion.charAt(i) == '*' && solucion.charAt(i) == '*');
            }
        }
        return resultado;
    }

    public double getPuntuacion(double puntuacionMaxima, String correccion, boolean puntuacionNegativaPermitida) {
        double puntuacion = 0;
        double numeroPreguntas = getNumeroPreguntas();
        double opciones = 0;
        double opcionesValidas = 0;
        double aciertos = 0;
        double errores = 0;
        double respuestas = 0;
        double valorPregunta = (puntuacionMaxima / numeroPreguntas);
        String codificacion = getCodificacion();
        if (mismaEstructura(correccion)) {
            for (int i = 0; i < codificacion.length(); i++) {
                switch (correccion.charAt(i)) {
                    case 'T':
                        opciones++;
                        respuestas++;
                        aciertos++;
                        opcionesValidas++;
                        break;
                    case 't':
                        opciones++;
                        opcionesValidas++;
                        break;
                    case 'F':
                        opciones++;
                        respuestas++;
                        errores++;
                        break;
                    case 'f':
                        opciones++;
                        break;
                    case '*':
                        if (opcionesValidas > 0 && respuestas > 0) {
                            puntuacion += (valorPregunta * (aciertos / opcionesValidas) - (valorPregunta * (errores / (opciones - 1))));
                        }
                        opciones = 0;
                        opcionesValidas = 0;
                        respuestas = 0;
                        aciertos = 0;
                        errores = 0;
                        break;
                }
            }
        }
        if (!puntuacionNegativaPermitida) puntuacion = Math.max(0, puntuacion);
        return puntuacion;
    }
}
